import WS from 'ws';
import { v4 as uuidv4 } from 'uuid';

interface ElevenLabsConfig {
    voice_id: string;
    model_id: string;
    output_format: string;
    optimize_streaming_latency: number;
    language_code: string;
    sync_alignment?: boolean;
    inactivity_timeout?: number; // max 100 seconds
}

function getEndpoint(config: ElevenLabsConfig): string {
    const baseUrl = `wss://api.elevenlabs.io/v1/text-to-speech/${config.voice_id}/stream-input`;
    const queryParams = new URLSearchParams();

    for (const key in config) {
        if (config.hasOwnProperty(key) && key !== 'voice_id') {
            queryParams.append(key, config[key as keyof ElevenLabsConfig] as string);
        }
    }

    return `${baseUrl}?${queryParams.toString()}`;
}

// function onMessage(data: Object)
type OnMessage = (data: any, randomUUID: string) => void;

async function connect(config: ElevenLabsConfig, onMessage: OnMessage): Promise<{ socket: WS; randomUUID: string }> {
    return new Promise((resolve, reject) => {
        const randomUUID = uuidv4();

        const endpoint = getEndpoint(config);
        const socket = new WS(endpoint, {
            headers: {
                'xi-api-key': process.env.ELEVENLABS_API_KEY,
            },
        });

        socket.on('open', () => {
            console.log('Connected to Eleven Labs');
            resolve({ socket, randomUUID });
        });

        socket.on('error', (err) => {
            console.error('Error connecting to Eleven Labs:', err);
            reject(err);
        });

        socket.on('message', (data) => {
            if (data instanceof Buffer) {
                const text = new TextDecoder().decode(new Uint8Array(data));

                const obj = JSON.parse(text);
                onMessage(obj, randomUUID);
                console.log('Received data from Eleven Labs WebSocket:', obj.isFinal);
            } else {
                console.log('Received data from Eleven Labs WebSocket:', data, typeof data);
            }
        });
    });
}

/*{
  "text": "This is a sample text ",
  "voice_settings": {
    "stability": 0.8,
    "similarity_boost": 0.8
  },
  "generation_config": {
    "chunk_length_schedule": [120, 160, 250, 290]
  },
  "xi_api_key": "<XI API Key>",
  "authorization": "Bearer <Authorization Token>"
}*/
interface GenerateParams {
    text: string;
    flush?: boolean;
    voice_settings?: {
        stability?: number;
        similarity_boost?: number;
        style?: number;
        use_speaker_boost?: boolean;
    };
    generation_config?: {
        chunk_length_schedule: number[];
    };
}

function generate(socket: WS, param: GenerateParams) {
    socket.send(JSON.stringify({ ...param, generation_config: { chunk_length_schedule: [120, 160, 250, 290] } }));
}

export default {
    connect,
    generate,
};
